#include "getCoverage.h"

#define DEBUG 0

typedef struct OFFSET_NODE{
    char *id;  /* read id */
    off_t offset;
    seqlen_t qvseqlen;
    struct OFFSET_NODE *next;
} offsetNode;

typedef struct OFFSET_HEAD{
    int num;
    struct OFFSET_NODE *next;
    struct OFFSET_NODE *end;
} offsetHead;


offsetNode * insertOffset(offsetHead *head)
{
  offsetNode *n = (offsetNode *) pool_malloc(sizeof(offsetNode));
  n->next = NULL;

  if(head->next == NULL)
  { 
    // if this is the first element inserted
    head->next = n;
    head->end = n;
  }
  else
  {
    head->end->next = n;
  }

  head->end = n;
  head->num++;

  return n;
}


int seqnamecmp(const void * a, const void * b)
{
  const Offset * ao = (const Offset *) a;
  const Offset * bo = (const Offset *) b;

  return strcmp(ao->id, bo->id);
}

/*
  indexSeqIDs()
  read a .qual file and create an index of seq_id offsets in the file
*/
void indexSeqIDs(FILE * f,
                 char *file,
                 Offset **qvIndex,
                 int *numOfQVreads,
                 char useIndex)
{
  FILE *f_idx = NULL;
  char fileindex [2*MAXFILENAME];

  offsetHead *head;
  Offset *ind;
  offsetNode *n;
  char c;
  int i;
  int count = 0;
  char seq_name[MAXSEQNAME];
  off_t offset;
  int qvseqlen;
  int indigit;

  head  = (offsetHead *) pool_malloc(sizeof(offsetHead));
  head->num = 0;
  head->next = NULL;
  head->end = NULL;
 
  if (useIndex)
  {
    strcpy(fileindex, file);
    strcat(fileindex, ".idx");
    f_idx = fopen(fileindex, "r"); // Don't exit if not found
  }

  if (f_idx)
  {
    while(fscanf(f_idx,"%s %d %lld\n", seq_name, &qvseqlen, &offset) != EOF)
    {
      // fprintf(stderr, "Loaded %s %d %lld\n", seq_name, qvseqlen, offset);

      n = insertOffset(head);
      n->qvseqlen = qvseqlen;
      n->offset = offset;
      n->id = (char *)newmem2(strlen(seq_name)+1,sizeof(char),__FILE__,__LINE__); 
      strcpy(n->id,seq_name);
    }
  }
  else
  {
    if (useIndex)
    {
      f_idx = fOpen(fileindex, "w");
    }

    c = fgetc(f); 

    // Move to the first record
    while (c != EOF && c != '>') { c = fgetc(f); }

    // Read seq_ids and offsets from the .qual file
    while( c != EOF )
    {
      count++;
      qvseqlen = 0;

      // offset records the first letter of the seq_name
      offset = ftello(f);  

      /* Read the sequence name into the name array */
      c = fgetc(f); /* read the first character of seq_id */
      for (i=0; !isspace(c); i++) 
      {
        seq_name[i] = c;
        c = fgetc(f);
      }
      seq_name[i] = '\0';  /* nullify the newline */

      while (!(c == '\n' || c == EOF))
      {
        c = fgetc(f); // Allow for trailing blanks, other fields
      }

      indigit = 0;
      while ((c=fgetc(f)) != EOF && c != '>')
      {
        if (isdigit(c)) { if (!indigit) { qvseqlen++; indigit = 1; } }
        else { indigit = 0; }
      }

      n = insertOffset(head);

      // These should persist a pool flush
      n->offset = offset;
      n->qvseqlen = qvseqlen; // qvseqlen is exactly correct
      n->id = (char *)newmem2(strlen(seq_name)+1,sizeof(char),__FILE__,__LINE__); 
      strcpy(n->id,seq_name);

      if (f_idx)
      {
        fprintf(f_idx, "%s %d %lld\n", seq_name, qvseqlen, offset);
      }
    }

    fflush(f_idx);
  }

  if (f_idx) { fclose(f_idx); }

  if(DEBUG) printf("head->num:%d\n",head->num);

  // Flatten the linked list into an array
      
  ind = (Offset *) newmem2(head->num, sizeof(Offset),__FILE__,__LINE__);
  n = head->next;
  i = 0;
  while(n != NULL)
  {
    ind[i].qvseqlen = n->qvseqlen;
    ind[i].offset = n->offset;
    ind[i].id = n->id; // just save the pointer
    n = n->next;
    i++;
  }

  qsort(ind, head->num, sizeof(Offset), seqnamecmp);

  if(DEBUG)
  {
    for(i = 0; i < head->num; i++)
    {
      printf("%s %lld\n",ind[i].id,ind[i].offset);
    }
    printf("\n");
  }
  
  *qvIndex = ind;
  *numOfQVreads = head->num;

  pool_flush(); // Flush the linked list
}
