#include "AmbiguityWriter.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  string infile;

  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);
    string dependencies = "xerces";
    string helptext = 
"Prints \n"
"  0. unambiguous consensus (ptw)\n"
"  1. Churchill-Waterman consenus (cw)\n"
"  2. conic consensus (conic)\n"
"  3. high quality consensus (hq)\n"
"\n"
"   Usage: sliceAmbiguity [options] slice.xml\n"
"\n"
"     slice.xml Slice XML file\n"
"\n"
 + RequestManager::getOptionHelp();

    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    AmbiguityWriter::handleOptions(tf);
    AmbiguityWriter manager;

    // Read the input filename
    if ((infile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify an input file", "sliceAmbiguity");
    }

    // Process the file
    retval = manager.processFile(infile, cerr);
  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
